# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from hipster_api.management.helpers import search_api
from hipster_api.management.helpers import get_dict
from hipster_api.management.helpers import create_html


class Command(BaseCommand):
    help = u'Зборка api для проекта'

    def handle(self, *args, **kwargs):
        list(map(create_html.render_doc,
                 list(filter(lambda x: x is not None, map(get_dict.format_method_urls, search_api.search_ulrs())))))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from hipster_api.fields.base import Field
from hipster_api.fields.str import StringList as _StringList


class Integer(Field):
    def to_python(self):
        value = super(Integer, self).to_python()

        try:
            value = int(value)
        except ValueError:
            value = self.default

        self.setitem(value)
        return value


class IntegerLarger(Integer):

    def __init__(self, larger, glt=False, **kwargs):
        self.larger = larger
        self.glt = glt
        super(IntegerLarger, self).__init__(**kwargs)

    def to_python(self):
        value = super(IntegerLarger, self).to_python()
        if self.glt:
            if value < self.larger:
                value = self.default
        elif value <= self.larger:
            value = self.default
        self.value = value
        return value


class IntegerList(_StringList):
    def to_python(self):
        value = super(IntegerList, self).to_python()
        try:
            value = list(map(lambda x: int(x), value))
        except ValueError:
            self.value = self.default
            return self.to_python()
        self.value = value
        return value



# -*- coding: utf-8 -*-
import os
from django.conf import settings

DIR = os.path.join(os.path.dirname(__file__), 'templates')


def load_default():
    setattr(settings, 'TEMPLATE_DIRS',  getattr(settings, 'TEMPLATE_DIRS', ()) + (DIR, ))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from hipster_api.fields.base import Field


class String(Field):
    def __init__(self, default='', **kwargs):
        kwargs['default'] = default
        super(String, self).__init__(**kwargs)

    def to_python(self):
        value = super(String, self).to_python()
        try:
            value = str(value)
        except ValueError:
            value = str(self.default)

        self.value = value
        return value


class StringList(String):

    def __init__(self, separator=',', **kwargs):
        self.separator = separator
        super(StringList, self).__init__(**kwargs)

    def to_python(self):
        value = super(StringList, self).to_python()
        value = value.replace(' ', '').split(self.separator)
        value = value if all(value) else list()
        self.value = value
        return self.value


class FieldsListResponse(StringList):

    global_fields = ['password', 'pwd']

    def exclude_global_fields(self, request):
        self.setitem(list(filter(
            lambda field: len(list(
                set(self.global_fields) & set(field.split('__'))
            )) == 0, self.value
        )))

    rules = (
        exclude_global_fields,
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class Field(object):

    def __init__(
            self,
            verbose_name=None,
            default=None,
            rules=None,
            methods=None
    ):
        self.__rules = (rules or tuple()) + getattr(self.__class__, 'rules', tuple())
        self.default = default
        self.value = default
        self.methods = methods or ['get', 'put', 'post', 'delete']
        self.verbose_name = verbose_name or self.__class__.__name__

    def to_python(self):
        return self.value

    def to_rules(self, request):
        tuple(map(lambda fn: fn(self, request), self.__rules))
        return self

    def setitem(self, value):
        self.value = value
        return self



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import io
import os
import shutil
from django.template.loader import render_to_string
from django.conf import settings as dj_settings


def rm_r(path):
    if os.path.isdir(path):
        shutil.rmtree(path)
    elif os.path.exists(path):
        os.remove(path)

rm_r(os.path.join(dj_settings.TEMPLATE_DIRS[0], 'docs', 'menu2.html'))
rm_r(os.path.join(dj_settings.TEMPLATE_DIRS[0], 'docs', 'files'))
os.mkdir(os.path.join(dj_settings.TEMPLATE_DIRS[0], 'docs', 'files'))


def render_doc(item):
    url = '/docs/api%s' % item['url'].replace('.json', '')
    body = render_to_string('docs/skeleton/body2.html', item)
    file_name = 'docs_api%s' % item['url'].replace('/', '_').replace('.json', '.html')
    file_name = os.path.join(dj_settings.TEMPLATE_DIRS[0], 'docs', 'files', file_name)
    f = io.open(file_name, 'w+', encoding='utf8')
    f.write(body)
    f.close()

    menu_patch = os.path.join(dj_settings.TEMPLATE_DIRS[0], 'docs', 'menu2.html')
    f = io.open(menu_patch, 'a+', encoding='utf8')
    f.write(render_to_string('docs/skeleton/menu2.html', {'url': url, 'name': item['text'], 'class': item['class']}))
    f.close()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.utils.encoding import smart_unicode
from hipster_api.fields import Field


def parse_doc_block(docs):
    docs = smart_unicode(docs)
    blocks = {
        'text': None
    }

    if not docs:
        return blocks

    text = False
    for doc in docs.split('\n'):
        if not doc.strip():
            continue
        doc = doc.strip()

        if doc[0] != ':' and not text:
            blocks['text'] = u"%s\n%s".strip() % (blocks['text'], doc) if blocks['text'] else doc
        else:
            text = True
            val = doc[1:-1].split(' ')
            if len(val) == 1:
                key = val[0]
                val = val[0]
            else:
                key = val.pop(0)
                val = " ".join(val)
            blocks[key] = val

    return blocks


def parse_filed_urls(key, obj):
    if issubclass(obj.__class__, Field):
        struct = {
            'name': key,
            'default': obj.default if obj.default is not None else 'null',
            'methods': obj.methods,
            'type': type(obj.default).__name__,
            'verbose_name': obj.verbose_name
        }
        return struct
    return None


def format_method_urls(item):
    url, cls = item

    text = parse_doc_block(cls.__doc__)['text']
    if text == 'None':
        return None
    item = {'url': url, 'text': text, 'class': cls.__name__}
    methods = {}
    fields = []
    for el in cls.__dict__.itervalues():
        try:

            if el.__name__.lower() in ['get', 'put', 'post', 'delete']:
                methods[el.__name__.lower()] = parse_doc_block(el.__doc__)
            elif el.__name__ == 'Fields':
                fields = list(filter(
                    lambda x: x is not None, [parse_filed_urls(x, y) for x, y in el.__dict__.iteritems()]))

        except AttributeError:
            pass

    def methods_merge_fields_urls(method):
        methods[method]['fields'] = list(filter(lambda x: method in x['methods'], fields))

    list(map(methods_merge_fields_urls, methods.keys()))
    item['methods'] = methods

    return item



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.conf import settings
from django.core.urlresolvers import RegexURLResolver
from django.utils.importlib import import_module


def url_pattern(url_pattern_regex, url_parent=''):
    callback = None
    url = url_parent
    if url and url[0] == '^':
        url = url[1:]

    url_parent = url_pattern_regex._regex

    try:
        callback = url_pattern_regex.callback.__dict__
        if 'cls' in callback and all(list(filter(lambda x: x.__name__ == 'APIView', callback['cls'].mro()))):
            callback = callback['cls']
        else:
            callback = None
    except AttributeError:
        pass

    if url_parent and url_parent[0] == '^':
        url_parent = url_parent[1:]

    url = '/%s%s' % (url, url_parent)
    url = url.replace('\.(?P<format>\w+)$', '.json')

    return url, callback


def search_ulrs():
    urls = import_module(settings.ROOT_URLCONF)
    urls_pars = []
    for url in urls.urlpatterns:
        if isinstance(url, RegexURLResolver):
            urls_pars += list(map(lambda x: url_pattern(x, url._regex), url.url_patterns))
        else:
            urls_pars += [url_pattern(url, '')]
    return list(filter(lambda x: x[1] is not None, urls_pars))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.http import QueryDict
from rest_framework.views import APIView
from hipster_api.fields.base import Field


class HManager(object):

    def __init__(self):
        self.__fields = {}
        self.__request = None

    def __setitem__(self, key, value):
        if key == '__request':
            self.__request = value
        else:
            self.__fields[key] = value

    def __getattr__(self, key):
        if key not in ['_HManager__fields', '_HManager__request']:
            instance = self.__fields.get(key)
            instance.to_rules(self.__request)
            return instance.value


class HView(APIView):

    def __init__(self, **kwargs):
        self.objects = HManager()
        super(HView, self).__init__(**kwargs)

    def initialize_request(self, request, *args, **kwargs):
        new_request = super(HView, self).initialize_request(request, *args, **kwargs)

        init_dict = QueryDict('', mutable=True)
        post = getattr(new_request, 'POST', QueryDict(''))
        data = getattr(new_request, 'data', False)
        init_dict.update(getattr(new_request, 'GET', QueryDict('')))

        if data:
            init_dict.update(data)
        elif len(post) > 0:
            init_dict.update(post)

        self.__set_media(init_dict)
        self.objects['__request'] = new_request

        return new_request

    def __set_media(self, data):
        for key, val in self.__class__.__dict__['Fields'].__dict__.items():
            if isinstance(val, Field):
                val.setitem(data.get(key, val.default)).to_python()
                self.objects[key] = val



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import codecs
import os
from distutils.core import setup
from setuptools import find_packages


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='django_hipster_api',
    version='1.6.1',
    packages=find_packages(),
    long_description=read("README.md"),
    install_requires='',
    url='https://github.com/RustoriaRu/hipster_api',
    license='MIT',
    author='vir-mir',
    keywords='django rest framework',
    author_email='virmir49@gmail.com',
    description='wrapper django rest framework',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
